# - encoding: utf8 - 
#
# Copyright © 2010 Alexey Grunichev
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import rb, rhythmdb
import gobject, gtk
import vkapi_helpers
import VkBaseSource

class VkontakteSearch:
    def __init__(self, search_term, db, entry_type, vk_queue):
        self.vk_queue = vk_queue
        self.search_term = search_term
        self.db = db
        self.entry_type = entry_type
        self.query_model = rhythmdb.QueryModel()
        self.complete = False
        self.error = None
        self.ready_result_count = 0
    
    # Returns true if the search is complete, false if it isn't
    def is_complete(self):
        return self.complete
    
    def add_entry(self, result):
        # Create the db entry and add it to the query model for this search
        self.query_model.add_entry(vkapi_helpers.entry_from_api_response(self.db, self.entry_type,result), -1)
        
        self.ready_result_count += 1

    def search_ok(self, data):
        self.complete = True
        print data
        for song in data[1:]:
            self.add_entry(song)
        
    def search_error(self, code, msg, rp):
        self.error = "Error %d, %s, %s" % (code, msg, rp)
         
    # Starts searching
    def start(self):
        event = self.vk_queue.add('audio.search',
                                  {'q': self.search_term, 'count': 1024},
                                  fok = self.search_ok,
                                  ferror = self.search_error)


class SearchSource(rb.Source, VkBaseSource.VkBaseSource):
    plugin = gobject.property()
    
    def __init__(self):
        rb.Source.__init__(self)
        self.initialised = False
        
        

    def initialise(self):
        shell = self.props.shell
        self.entry_view = rb.EntryView(shell.props.db, shell.get_player(), "", True, False)
    
        query_model = rhythmdb.QueryModel()
        self.props.query_model = query_model
        self.treeview = None
        self.entry_view.set_model(query_model)
        self.entry_view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.entry_view.set_shadow_type(gtk.SHADOW_IN)
  
        self.entry_view.append_column(rb.ENTRY_VIEW_COL_TITLE, True)
        self.entry_view.append_column(rb.ENTRY_VIEW_COL_ARTIST, True)
        self.entry_view.append_column(rb.ENTRY_VIEW_COL_DURATION, True)
        #self.entry_view.append_column(rb.ENTRY_VIEW_COL_ALBUM, True)

        self.add_ui_actions()
        
    def add_ui_actions(self):
  
        # Set up the search bar and button UI. This could probably be done in a better way.
        search_entry = gtk.combo_box_entry_new_text()
        self.search_button = gtk.Button("Search")
        alignment = gtk.Alignment()
        alignment.add(self.search_button)
        hbox = gtk.HBox()
        hbox.pack_start(search_entry)
        hbox.pack_start(alignment)
        hbox.set_child_packing(search_entry, True, True, 0, gtk.PACK_START)
        hbox.set_child_packing(alignment, True, True, 2, gtk.PACK_START)
        vbox = gtk.VBox()
        vbox.pack_start(hbox)
        vbox.set_child_packing(hbox, False, False, 2, gtk.PACK_START)
        vbox.pack_start(self.entry_view)
        self.add(vbox)
        self.show_all()
        
        self.search_button.connect("clicked", self.on_search_button_clicked, search_entry)
        search_entry.child.set_activates_default(True)
        search_entry.connect("changed", self.on_search_entry_changed)
        self.search_button.set_flags(gtk.CAN_DEFAULT)
        
        self.searches = {} # Dictionary of searches, with the search term as keys
        self.current_search = "" # The search term of the search results currently being shown
        self.is_adding_fav = False
        self.added_fav_message = None
        
        
        
        
        self.actions_list = (
            #([0]'actionname', [1]_('Action Human name'), [2]_('Action tooltip'), [3]'action-icon', [4] actionfunction, [5] is_per_item,  [6]is_in_toolbar, [7] is_toogle_action,),
            #('', _(''), _(''), '', , , , ),
            ('VkontakteFav', _('_Favorite'), _('Makes favorite in "My Music" in vkontakte'), 'gtk-about',  self.add_to_fav, True, True, False ),
            ('VkontakteKillDuplicates', _('Kill _Duplicates'), _('Detectes and does not shows duplicates at search'), 'gtk-copy', self.add_to_fav, False, True, True ),
#            ('', _(''), _(''), '', , , , ),
#            ('', _(''), _(''), '', , , , ),
#            ('', _(''), _(''), '', , , , ),
#            ('', _(''), _(''), '', , , , ),
#            ('', _(''), _(''), '', , , , ),
#            ('', _(''), _
             )
        
        self.popup_ui = '''<ui> 
  <popup name="VkontakteSourceViewPopup">
    <menuitem name="AddToQueueLibraryPopup" action="AddToQueue"/>
    <menuitem name="VkontakteFav" action="VkontakteFav"/>
    <separator/>
    <menuitem name="PropertiesLibraryPopup" action="MusicProperties"/>
  </popup>
  </ui>'''

        self.initialise_actions()
        
        self.initialised = True
        
    def do_impl_get_entry_view(self):
        return self.entry_view
    
    def do_impl_activate(self):
        if not self.initialised:
            self.initialise()
        self.search_button.grab_default()
            
    def do_impl_get_status(self):
        print self.added_fav_message
        if self.added_fav_message is not None:
            if self.is_adding_fav:
                addfavst = -1
            else:
                addfavst = 1
            added_fav_message = self.added_fav_message
            #self.added_fav_message = None
            return (added_fav_message, "", addfavst)
        if self.current_search:
            if self.searches[self.current_search].is_complete():
                return (self.props.query_model.compute_status_normal("Found %d result", "Found %d results"), "", 1)
            elif self.searches[self.current_search].error != None:
                return ("Api fuction return error: %s"%self.searches[self.current_search].error, "", 1)
            else:
                return ("Searching for \"{0}\"".format(self.current_search), "", -1)
            
        else:
            return ("", "", 1)
            
    def do_impl_delete_thyself(self):
        if self.initialised:
            self.props.shell.props.db.entry_delete_by_type(self.props.entry_type)
        rb.Source.do_impl_delete_thyself(self)
        
    def do_impl_can_add_to_queue(self):
        return True
        
    def do_impl_can_pause(self):
        return True
        
    def on_search_button_clicked(self, button, entry):
        # Only do anything if there is text in the search entry
        if entry.get_active_text():
            # If this term has never been searched before then create a new search object, add it to the dictionary, and start the search. 
            if not entry.get_active_text() in self.searches:
                self.searches[entry.get_active_text()] = VkontakteSearch(entry.get_active_text(), self.props.shell.props.db, self.props.entry_type, self.props.plugin.vk_queue)
                # Start the search asynchronously
                self.searches[entry.get_active_text()].start()
                entry.prepend_text(entry.get_active_text())
            
            # Update the entry view and source so the display the query model relevant to the current search
            self.current_search = entry.get_active_text()
            self.props.query_model = self.searches[self.current_search].query_model
            self.entry_view.set_model(self.props.query_model)
            
            
    def on_search_entry_changed(self, entry):
        if entry.get_active_text() in self.searches:
            self.current_search = entry.get_active_text()
            self.props.query_model = self.searches[self.current_search].query_model
            self.entry_view.set_model(self.props.query_model)
    

    def do_impl_show_entry_popup(self,  some_int, some_bool):
        self.show_source_popup ("/VkontakteSourceViewPopup")
        
    
    def do_impl_get_ui_actions(self):
        return self.ui_actions
    
    def filter_dup_entry(self, entry, data=None):
        print entry.get_string(rhythmdb.PROP_ALBUM)
        entry.set(rhythmdb.PROP_HIDDEN, True)
        print "hd"
        
    
    def toggle_kill_duplicates(self):
        self.entry_view.entry_foreach_by_type(self.props.entry_type, self.filter_dup_entry, None)
        #        if self.props.plugin.action_group.get_action("VkontakteFav").get_active():
        #            for entry in self.props.shell.props.db:
        #                print entry
        #        else:
        #            for entry in self.props.shell.props.db:
        #                print entry
        #                print "deactivated"
        
        
    def add_ok(self, data):
        self.added_fav_message = "Song added to vkontakte's user's favorites"
        self.is_adding_fav = False
        self.notify_status_changed()
        
    def add_error(self):
        self.added_fav_message = "ERROR: Can't add song to vkontakte's user's favorites"
        self.is_adding_fav = False
        self.notify_status_changed()
        
    def add_to_fav(self):
        shell = self.props.shell
        if self.entry_view.have_selection():
            self.is_adding_fav = True
            self.added_fav_message = "Adding to vkontakte's favorites..."
            self.notify_status_changed()
            for entry in self.entry_view.get_selected_entries():
                res = vkapi_helpers.api_response_from_entry(shell.props.db, entry)
                event = self.props.plugin.vk_queue.add('audio.add', {'aid':res['aid'], 'oid':res['owner_id']}, fok=self.add_ok, ferror=self.add_error)
        else:
            self.added_fav_message = "Adding to vkontakte's favorites failed: select songs first!"


gobject.type_register(SearchSource)
