# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Vadim Alimguzhin <darth@darth.su>, Nazar Vinnichuk <UX.ISCo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

import rb, rhythmdb, gobject, gtk, vkapi
from ListsSource import ListsSource
from SearchSource import SearchSource


APP_ID   = 1922565
SETTINGS = 10


class VkEntryType(rhythmdb.EntryType):
	def __init__(self):
		rhythmdb.EntryType.__init__(self, name = 'VkEntryType')


class VkontaktePlugin(rb.Plugin):
    def _mk_source(self, source_type, name, icon_file):
    
        width, height = gtk.icon_size_lookup(gtk.ICON_SIZE_LARGE_TOOLBAR)
        icon = gtk.gdk.pixbuf_new_from_file_at_size(self.find_file(icon_file), width, height)
        
        source = gobject.new(source_type,
                             icon         = icon,
                             name         = name,
                             entry_type   = self.entry_type,
                             source_group = self.source_group,
                             shell        = self.shell,
                             plugin       = self)
        
        self.shell.append_source(source, None)
        
        return source
    
    
    def activate(self, shell):
        self.vk_session = vkapi.VkSession(APP_ID, SETTINGS)
        self.vk_query   = vkapi.VkQuery(self.vk_session)
        self.vk_queue   = vkapi.VkQueue(self.vk_query, 5, 1)
        
        self.shell = shell
        self.db    = shell.props.db
        
        self.source_group = rb.rb_source_group_get_by_name('vkontakte')
        
        if self.source_group is None:
            self.source_group = rb.rb_source_group_register('vkontakte', _('VKontakte'),
                                                            rb.SOURCE_GROUP_CATEGORY_TRANSIENT)
        
        try:
            self.entry_type = VkEntryType()
            self.db.register_entry_type(self.entry_type)
        except NotImplementedError:
            self.entry_type = self.db.entry_register_type('VkEntryType')
        finally:
            self.entry_type.can_sync_metadata = True
            self.entry_type.sync_metadata     = None
            self.entry_type.category          = rhythmdb.ENTRY_STREAM
        
        self.lists_source = self._mk_source(ListsSource,
                                            _('My Music'),
                                            'icons/vkontakte_library.png')
        
        self.search_source = self._mk_source(SearchSource,
                                             _('Search'),
                                             'icons/vkontakte_search.png')
        
        print "Vkontakte plugin activated"
    
    
    def deactivate(self, shell):
        #manager = shell.get_player().get_property('ui-manager')
        #manager.remove_ui (self.ui_id)
        #manager.remove_action_group(self.action_group)
        #self.action_group = None
        
        self.db.entry_delete_by_type(self.entry_type)
        self.db.commit()
        
        self.lists_source.delete_thyself()
        self.search_source.delete_thyself()
        
        del self.search_source
        del self.lists_source
        del self.entry_type
        del self.source_group
        del self.db
        del self.shell
        del self.vk_queue
        del self.vk_query
        del self.vk_session
        
        print "Vkontakte plugin deactivated"

