# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Vadim Alimguzhin <darth@darth.su>, Nazar Vinnichuk <UX.ISCo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

import re, htmlentitydefs, rhythmdb, json


##
# Removes HTML or XML character references and entities from a text string.
#
# @param text The HTML (or XML) source text.
# @return The plain text, as a Unicode string, if necessary.

def unescape(text):
    def fixup(m):
        text = m.group(0)
        
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        
        return text # leave as is
    
    return re.sub('^\s+|\s+$', '', re.sub("&#?\w+;", fixup, text))


FIELDS = {'title'    : (rhythmdb.PROP_TITLE,    unescape),
          'artist'   : (rhythmdb.PROP_ARTIST,   unescape),
          'duration' : (rhythmdb.PROP_DURATION, int)}

METADATA_FIELD = rhythmdb.PROP_GENRE

def entry_from_api_response(db, entry_type, vk_record):
    url = vk_record['url']
    
    entry = db.entry_lookup_by_location(url)
    if entry == None:
        entry = db.entry_new(entry_type, url)
    
    db.set(entry, METADATA_FIELD, json.dumps(vk_record))
    
    for vk_field in FIELDS:
        if vk_field in vk_record:
            try:
                db.set(entry, FIELDS[vk_field][0], FIELDS[vk_field][1](vk_record[vk_field]))
            except:
                pass
    
    return entry


def api_response_from_entry(db, entry):
    try:
        return json.loads(db.entry_get(entry, METADATA_FIELD))
    except:
        return None

